@@ -9,8 +9,8 @@ __docformat__ = 'restructuredtext en'
 import textwrap, os, shlex, subprocess, glob, shutil, re, sys, json
 from collections import namedtuple
 
-from setup import Command, islinux, isbsd, isosx, SRC, iswindows, __version__
-isunix = islinux or isosx or isbsd
+from setup import Command, islinux, isbsd, isosx, ishaiku, SRC, iswindows, __version__
+isunix = islinux or isosx or isbsd or ishaiku
 
 py_lib = os.path.join(sys.prefix, 'libs', 'python%d%d.lib' % sys.version_info[:2])
 
@@ -81,7 +81,7 @@ def is_ext_allowed(ext):
     only = ext.get('only', '')
     if only:
         only = only.split()
-        q = 'windows' if iswindows else 'osx' if isosx else 'bsd' if isbsd else 'linux'
+        q = 'windows' if iswindows else 'osx' if isosx else 'bsd' if isbsd else 'haiku' if ishaiku else 'linux'
         return q in only
     return True
 
@@ -100,6 +100,8 @@ def parse_extension(ext):
             ans = ext.pop('osx_' + k, ans)
         elif isbsd:
             ans = ext.pop('bsd_' + k, ans)
+        elif ishaiku:
+            ans = ext.pop('haiku_' + k, ans)
         else:
             ans = ext.pop('linux_' + k, ans)
         return ans
@@ -156,6 +158,12 @@ def init_env():
         cflags.append('-I'+sysconfig.get_python_inc())
         ldflags.append('-lpython'+sysconfig.get_python_version())
 
+    if ishaiku:
+        cflags.append('-lpthread')
+        ldflags.append('-shared')
+        cflags.append('-I'+sysconfig.get_python_inc())
+        ldflags.append('-lpython'+sysconfig.get_python_version())
+
     if isosx:
         cflags.append('-D_OSX')
         ldflags.extend('-bundle -undefined dynamic_lookup'.split())
@@ -329,7 +337,7 @@ class Build(Command):
 
     def build_headless(self):
         from setup.parallel_build import cpu_count
-        if iswindows or isosx:
+        if iswindows or isosx or ishaiku:
             return  # Dont have headless operation on these platforms
         from setup.build_environment import glib_flags, fontconfig_flags, ft_inc_dirs, QMAKE
         from PyQt5.QtCore import QT_VERSION
